@@ -14,13 +14,13 @@ type Callback<'a> = dyn Fn(Box<BookTickerEvent>) -> () + Send + 'static;
 pub struct BinanceTickData {
     pub A: String,
     pub B: String,
-    pub E: f64,
-    pub T: f64,
+    pub E: i64,
+    pub T: i64,
     pub a: String,
     pub b: String,
     pub e: String,
     pub s: String,
-    pub u: u64,
+    pub u: i64,
 }
 
 #[derive(Debug, Deserialize)]
