@@ -1,24 +1,37 @@
-use std::collections::HashMap;
 use std::sync::Arc;
 
+use pyo3::prelude::*;
 use futures::{pin_mut, StreamExt};
 use futures::future::select;
-use pyo3::prelude::*;
-use serde_json::Value;
-// use futures_util::{future};
+use pyo3::types::{IntoPyDict, PyDict};
 use tokio::io::{AsyncReadExt, AsyncWriteExt};
 use tokio_tungstenite::{connect_async, tungstenite::protocol::Message};
 
-use crate::binance_connector::{BinanceListener, BinanceListenerOptions, BinanceTick};
+use crate::binance_connector::BinanceTick;
 use crate::python::models::TickData;
 
 #[allow(unused_variables)]
 mod binance_connector;
 mod python;
 
-#[pyclass]
-struct ExchangeListener {
-    // token: Option<String>,
+#[pyfunction]
+fn some(py: Python) -> PyResult<&PyAny> {
+    let tick = PyModule::import(py, "nautilus_trader.model.data.tick").unwrap();
+    let qt = tick.getattr("QuoteTick").unwrap();
+
+    let key_vals: Vec<(&str, PyObject)> = vec![
+        ("instrument_id", "BTCUSD-PERP.BINANCE".to_object(py)),
+        ("bid", "11".to_object(py)),
+        ("ask", "22".to_object(py)),
+        ("bid_size", "111".to_object(py)),
+        ("ask_size", "222".to_object(py)),
+        ("update_id", "3333333".to_object(py)),
+        ("ts_event", 4444444.to_object(py)),
+        ("ts_init", 5555555.to_object(py)),
+    ];
+    let dict = key_vals.into_py_dict(py);
+    let qt_instance = qt.call_method1("from_dict", (dict, )).expect("QT error");
+    Ok(qt_instance)
 }
 
 #[pyfunction]
@@ -29,6 +42,11 @@ fn rust_sleep(py: Python) -> PyResult<&PyAny> {
     })
 }
 
+#[pyclass]
+struct ExchangeListener {
+    // token: Option<String>,
+}
+
 #[pymethods]
 impl ExchangeListener {
     #[new]
@@ -38,9 +56,7 @@ impl ExchangeListener {
 
     fn subscribe<'a>(&'a self, py: Python<'a>, tickers: Vec<String>, callback: PyObject) -> PyResult<&PyAny> {
         pyo3_asyncio::tokio::future_into_py(py, async move {
-            // let it: BinanceListener = BinanceListener::new();
-            // let opts = BinanceListenerOptions::new(tickers);
-            let streams = tickers.iter().map(|t|format!("{t}@bookTicker")).collect::<Vec<String>>().join("/");
+            let streams = tickers.iter().map(|t| format!("{t}@bookTicker")).collect::<Vec<String>>().join("/");
             let url = url::Url::parse(format!("wss://fstream.binance.com/stream?streams={streams}").as_str()).unwrap();
             let (stdin_tx, stdin_rx) = futures_channel::mpsc::unbounded();
 
@@ -53,16 +69,34 @@ impl ExchangeListener {
             let cb = Arc::new(callback);
 
             let stdin_to_ws = stdin_rx.map(Ok).forward(write);
+
             let ws_to_stdout = {
                 read.for_each(|message| async {
-                    let guard = Python::acquire_gil();
-                    let py = guard.python();
-                    let data = message.unwrap().into_data();
-                    let tick: BinanceTick = serde_json::from_slice(&data).unwrap();
-                    let tick_data: TickData = tick.into();
-                    // dbg!(&tick_data);
+                    let _ = Python::with_gil(|py| {
+                        let data = message.unwrap().into_data();
+                        let tick: BinanceTick = serde_json::from_slice(&data).unwrap();
+                        // let tick_data: TickData = tick.into();
+                        let instrument_id = format!("{}-PERP.BINANCE", tick.data.s);
 
-                    &cb.clone().call(py, (tick_data, ), None).expect("wrong");
+                        // Form Tick
+                        let tick_module = PyModule::import(py, "nautilus_trader.model.data.tick").unwrap();
+                        let qt_class = tick_module.getattr("QuoteTick").unwrap();
+
+                        let key_vals: Vec<(&str, PyObject)> = vec![
+                            ("instrument_id", instrument_id.to_object(py)),
+                            ("bid", tick.data.b.to_object(py)),
+                            ("ask", tick.data.a.to_object(py)),
+                            ("bid_size", tick.data.B.to_object(py)),
+                            ("ask_size", tick.data.A.to_object(py)),
+                            ("update_id", tick.data.u.to_object(py)),
+                            ("ts_event", tick.data.E.to_object(py)),
+                            ("ts_init", tick.data.T.to_object(py)),
+                        ];
+                        let dict = key_vals.into_py_dict(py);
+                        let qt_instance = qt_class.call_method1("from_dict", (dict, )).expect("QT error");
+
+                        cb.clone().call(py, (qt_instance, ), None).expect("wrong");
+                    });
                 })
             };
 
@@ -84,14 +118,6 @@ impl ExchangeListener {
             Ok(())
         })
     }
-
-
-    fn sleep<'a>(&'a self, py: Python<'a>) -> PyResult<&PyAny> {
-        pyo3_asyncio::tokio::future_into_py(py, async {
-            tokio::time::sleep(std::time::Duration::from_secs(1)).await;
-            Ok(())
-        })
-    }
 }
 
 
@@ -101,6 +127,8 @@ impl ExchangeListener {
 #[pymodule]
 fn exchange_worker(_py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<ExchangeListener>()?;
+    m.add_class::<TickData>()?;
     m.add_function(wrap_pyfunction!(rust_sleep, m)?)?;
+    m.add_function(wrap_pyfunction!(some, m)?)?;
     Ok(())
 }
\ No newline at end of file
